﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Windows.Media;
using System.Xml.Linq;
using ff14bot;
using ff14bot.AClasses;
using ff14bot.BotBases;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.NeoProfiles;

namespace FateBotManagerPlugin
{
    public class FateBotManager : BotPlugin
    {
        #region Optional Overrides

        public override void OnInitialize()
        {
            LoadSettings();
        }

        public override void OnEnabled()
        {
            if (BotManager.Current != null && BotManager.Current.EnglishName != "Fate Bot")
            {
                Logging.Write(Colors.Red,
                    string.Format("Current BotBase is {0}. This plugin only works for FateBot.",
                        BotManager.Current.EnglishName));
                return;
            }
            TreeRoot.OnStart += OnBotStart;
            TreeRoot.OnStop += OnBotStop;
            GameEvents.OnLevelUp += OnPlayerLevelUp;

            if (TreeRoot.IsRunning)
            {
                AddHooks();
            }
        }

        public override void OnDisabled()
        {
            TreeRoot.OnStart -= OnBotStart;
            TreeRoot.OnStop -= OnBotStop;
            GameEvents.OnLevelUp -= OnPlayerLevelUp;
            RemoveHooks();
        }

        public override void OnShutdown()
        {
            TreeRoot.OnStart -= OnBotStart;
            TreeRoot.OnStop -= OnBotStop;
            GameEvents.OnLevelUp -= OnPlayerLevelUp;
            RemoveHooks();
        }

        #endregion

        #region Hook Logic

        private void AddHooks()
        {
            //TODO: Implement AddHooks
        }

        private void RemoveHooks()
        {
            //TODO: Implement RemoveHooks
        }

        #endregion

        #region Required Overrides

        public override string Author
        {
            get { return "Wheredidigo"; }
        }

        public override string Name
        {
            get { return "FateBot Manager"; }
        }

        public override Version Version
        {
            get { return new Version(1, 0, 0, 0); }
        }

        #endregion

        #region Event Handlers

        private void OnPlayerLevelUp(object sender, EventArgs e)
        {
            SetFateBotSettings();
        }

        private void OnBotStop(BotBase bot)
        {
            RemoveHooks();
        }

        private void OnBotStart(BotBase bot)
        {
            SetFateBotSettings();
            AddHooks();
        }

        #endregion

        #region Settings

        #region Set FateBot Settings Method

        private static void SetFateBotSettings()
        {
            FatebotSettings.Instance.BossEnabled = Settings.DoBossFates;
            FatebotSettings.Instance.BossPercentRequired = Settings.MinPercentCompleteForBoss;
            FatebotSettings.Instance.EscortEnabled = Settings.DoEscortFates;
            FatebotSettings.Instance.MaxLevel = Core.Me.ClassLevel + Settings.MaxLevelsAbove;
            FatebotSettings.Instance.MinLevel = Core.Me.ClassLevel - Settings.MaxLevelsBelow;
            FatebotSettings.Instance.ThisFateOnly = Settings.DoOnlyThisFate;
        }

        #endregion

        private static void LoadSettings()
        {
            var fileLocation = PluginManager.PluginDirectory + "\\FateBotManager\\FateBotManagerSettings.xml";
            XElement xmlSettings;
            if (File.Exists(fileLocation))
            {
                xmlSettings = XElement.Load(fileLocation);
            }
            else
            {
                xmlSettings = new XElement("Settings",
                    new XElement("DoBossFates", false),
                    new XElement("MinPercentCompleteForBoss", 0f),
                    new XElement("DoEscortFates", false),
                    new XElement("MaxLevelsAbove", 3),
                    new XElement("MaxLevelsBelow", 10),
                    new XElement("DoOnlyThisFate", string.Empty),
                    BuildTeleportLocations());
                xmlSettings.Save(fileLocation);
            }
            var settings = new FateBotManagerSettings
            {
                DoBossFates = (bool)xmlSettings.Element("DoBossFates"),
                MinPercentCompleteForBoss = (float)xmlSettings.Element("MinPercentCompleteForBoss"),
                DoEscortFates = (bool)xmlSettings.Element("DoEscortFates"),
                MaxLevelsAbove = (int)xmlSettings.Element("MaxLevelsAbove"),
                MaxLevelsBelow = (int)xmlSettings.Element("MaxLevelsBelow"),
                DoOnlyThisFate = (string)xmlSettings.Element("DoOnlyThisFate"),
                TeleportLocations = BuildTeleportLocations(xmlSettings.Element("TeleportLocations"))
            };
            Settings = settings;
        }

        private static FateBotManagerSettings _settings;

        private static FateBotManagerSettings Settings
        {
            get { return _settings ?? (_settings = new FateBotManagerSettings()); }
            set { _settings = value; }
        }

        private static dynamic BuildTeleportLocations(XElement locations = null)
        {
            if (locations == null)
            {
                var xmlRetval = new XElement("TeleportLocations");
                var textInfo = CultureInfo.CurrentCulture.TextInfo;

                foreach (var location in WorldManager.AvailableLocations)
                {
                    xmlRetval.Add(new XElement("TeleportLocation",
                        new XElement("Name", textInfo.ToTitleCase(location.Name)),
                        new XElement("AetheryteId", location.AetheryteId),
                        new XElement("ZoneId", location.ZoneId),
                        new XElement("JSTAtmaHour", 0),
                        new XElement("AtmaCount", 0),
                        new XElement("AtmaName", string.Empty)));
                }
                return xmlRetval;
            }
            return locations.Descendants("TeleportLocations").Select(location => new TeleportLocation
            {
                AetheryteId = (uint)location.Element("AetheryteId"),
                AtmaCount = (int)location.Element("AtmaCount"),
                AtmaName = (string)location.Element("AtmaName"),
                JstAtmaHour = (int)location.Element("JSTAtmaHour"),
                Name = (string)location.Element("Name"),
                ZoneId = (int)location.Element("ZoneId")
            }).ToList();
        }

        private static XElement BuildTeleportLocationsXml()
        {
            var xmlRetval = new XElement("TeleportLocations");
            var textInfo = CultureInfo.CurrentCulture.TextInfo;

            foreach (var location in WorldManager.AvailableLocations)
            {
                xmlRetval.Add(new XElement("TeleportLocation",
                    new XElement("Name", textInfo.ToTitleCase(location.Name)),
                    new XElement("AetheryteId", location.AetheryteId),
                    new XElement("ZoneId", location.ZoneId),
                    new XElement("JSTAtmaHour", 0),
                    new XElement("AtmaCount", 0),
                    new XElement("AtmaName", string.Empty)));
            }
            return xmlRetval;
        }

        private static IEnumerable<TeleportLocation> BuildTeleportLocationsEnumerable(XContainer locations)
        {
            return locations.Descendants("TeleportLocations").Select(location => new TeleportLocation
            {
                AetheryteId = (uint) location.Element("AetheryteId"),
                AtmaCount = (int) location.Element("AtmaCount"),
                AtmaName = (string) location.Element("AtmaName"),
                JstAtmaHour = (int) location.Element("JSTAtmaHour"),
                Name = (string) location.Element("Name"),
                ZoneId = (int) location.Element("ZoneId")
            }).ToList();
        }

        #endregion
    }
}